{-----------------------------------------------------------------------------
 Unit Name: fMain
 Author:    Lachlan Gemmell http://lachlan.gemmell.com/
 Purpose:   Editing a custom setting based on the default Delphi setting.
 History:
 12/27/04 - Received the file from Lachlan.
 01/02/05 - Updated to work with new ISettingCollection.
-----------------------------------------------------------------------------}
unit fEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, SettingCollection, Registry, ImgList, StdCtrls,
  ActnList, Buttons, XPStyleActnCtrls, ActnMan, ToolWin, ActnCtrls;

type
  TProfile = (pOriginal, pCustom);

  TProfileControls = record
    TreeView : TTreeView;
    ListBox : TListBox;
    Memo : TMemo;
    Registry : TRegistry;
  end;

  PProfileControls = ^TProfileControls;

  TfrmEditSetting2 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Splitter1: TSplitter;
    Panel4: TPanel;
    tvOriginal: TTreeView;
    tvCustom: TTreeView;
    ImageList1: TImageList;
    Splitter2: TSplitter;
    lbOriginal: TListBox;
    tbValueOps: TActionToolBar;
    lbCustom: TListBox;
    ActionList1: TActionList;
    actCopyRight: TAction;
    actDeleteRight: TAction;
    ActionManager1: TActionManager;
    ImageList2: TImageList;
    memOriginal: TMemo;
    memCustom: TMemo;
    lblOriginal: TLabel;
    lblCustom: TLabel;
    procedure TreeViewGetSelectedIndex(Sender: TObject; Node: TTreeNode);
    procedure TreeViewGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure actDeleteLeftExecute(Sender: TObject);
    procedure actDeleteRightExecute(Sender: TObject);
    procedure actCopyLeftExecute(Sender: TObject);
    procedure actCopyRightExecute(Sender: TObject);
    procedure lbCustomKeyPress(Sender: TObject; var Key: Char);
    procedure lbCustomDblClick(Sender: TObject);
    procedure lbCustomClick(Sender: TObject);
    procedure lbOriginalDblClick(Sender: TObject);
    procedure lbOriginalKeyPress(Sender: TObject; var Key: Char);
    procedure lbOriginalClick(Sender: TObject);
    procedure tvCustomChange(Sender: TObject; Node: TTreeNode);
    procedure tvOriginalChange(Sender: TObject; Node: TTreeNode);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FEditedSettingName,
    FCustomSettingRegPath,
    FDelphiRegPath : string;
    FProfile : array[TProfile] of TProfileControls;
    FRefreshing: Boolean;
    FSynchronising: Boolean;
    procedure DisplaySettings;
    procedure DisplaySubKeys(inProfile : TProfile; inNode: TTreeNode); overload;
    procedure DisplaySubKeys(inTreeView : TTreeView; inNode: TTreeNode; inRegistry : TRegistry); overload;
    procedure SynchroniseTreeViews(inSelectedProfile : TProfile);
    function RegKeyForSelected(inNode: TTreeNode): string;
    procedure DisplayValues(inProfile : TProfile; inPreserveItemIndex : Boolean = False);
    function NodesMatch(inSelectedNode, inOtherNode: TTreeNode): Boolean;
    procedure SynchroniseListBoxes(inSelectedProfile : TProfile);
    function MatchingRegKeyForSelected(inSelectedProfile: TProfile): string;
    procedure DeleteValue(inSelectedProfile : TProfile);
    procedure CopyValue(inSourceProfile : TProfile);
    procedure DisplayValue(inProfile : TProfile);
    procedure RetrieveProfileControls(inPrimaryProfile: TProfile;
          out outProfileControls1, outProfileControls2: PProfileControls);
    procedure SynchronizeAndDisplayValues;
  public
    procedure SetRegistryPath (const ADelphiPath, ASettingName, 
        ACustomSettingPath: string);
  end;

var
  frmEditSetting2: TfrmEditSetting2;

implementation

type
  EDSEException = class(Exception);

{$R *.dfm}

procedure TfrmEditSetting2.FormCreate(Sender: TObject);
begin
  FProfile[pOriginal].Registry := TRegistry.Create;
  FProfile[pOriginal].TreeView := tvOriginal;
  FProfile[pOriginal].ListBox := lbOriginal;
  FProfile[pOriginal].Memo := memOriginal;

  FProfile[pCustom].Registry := TRegistry.Create;
  FProfile[pCustom].TreeView := tvCustom;
  FProfile[pCustom].ListBox := lbCustom;
  FProfile[pCustom].Memo := memCustom;
end;

procedure TfrmEditSetting2.DisplaySettings;
const
  BORLAND_IDE_ROOT = '\Software\Borland';
var
  node: TTreeNode;
begin
  tvOriginal.Items.Clear;
  if not FProfile[pOriginal].Registry.OpenKey (FDelphiRegPath, False) then
    raise EDSEException.Create ('Cannot open root key ' + FDelphiRegPath);

  node := tvOriginal.Items.Add(nil, FProfile[pOriginal].Registry.CurrentPath);

  DisplaySubKeys(pOriginal, node);

  tvCustom.Items.Clear;

  if not FProfile[pCustom].Registry.OpenKey(FCustomSettingRegPath, False) then
    raise EDSEException.Create('Cannot open root key ' + FCustomSettingRegPath );

  node := tvCustom.Items.Add(nil, FProfile[pCustom].Registry.CurrentPath);

  DisplaySubKeys(pCustom, node);
end;

procedure TfrmEditSetting2.DisplaySubKeys(inProfile : TProfile; inNode : TTreeNode);
var
  selected : PProfileControls;
begin
  selected := @FProfile[inProfile];
  DisplaySubKeys(selected.TreeView, inNode, selected.Registry);
end;

procedure TfrmEditSetting2.DisplaySubKeys(inTreeView : TTreeView; inNode : TTreeNode; inRegistry : TRegistry);
var
  nodeKey: string;
  parentNode, node: TTreeNode;
  keys: TStringList;
  i: Integer;
begin
  nodeKey := inNode.Text;

  parentNode := inNode.Parent;
  while Assigned(parentNode) do begin
    nodeKey := parentNode.Text + '\' + nodeKey;
    parentNode := parentNode.Parent;
  end; // while

  keys := TStringList.Create;
  try
    if inRegistry.CurrentPath <> nodeKey then begin
      nodeKey := '\' + nodeKey;
      if not inRegistry.OpenKey(nodeKey, False) then
        raise EDSEException.Create('Cannot open the registry key ' + nodeKey);
    end; // if
    inRegistry.GetKeyNames(keys);
    for i := 0 to (keys.Count - 1) do begin
      node := inTreeView.Items.AddChild(inNode, keys[i]);

      DisplaySubKeys(inTreeView, node, inRegistry);
    end; // for
  finally
    FreeAndNil(keys);
  end; // finally
end;

procedure TfrmEditSetting2.FormShow(Sender: TObject);
begin
  FRefreshing := True;
  try
    DisplaySettings;
    lblOriginal.Caption := 'Original';
  finally
    FRefreshing := False;
  end; // finally
end;

procedure TfrmEditSetting2.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FProfile[pOriginal].Registry);
  FreeAndNil(FProfile[pCustom].Registry);
end;

procedure TfrmEditSetting2.tvOriginalChange(Sender: TObject;
  Node: TTreeNode);
begin
  if not FRefreshing and not FSynchronising then
    SynchroniseTreeViews(pOriginal);
  if not FSynchronising then
    DisplayValues(pOriginal);
end;

procedure TfrmEditSetting2.tvCustomChange(Sender: TObject;
  Node: TTreeNode);
begin
  if not FRefreshing and not FSynchronising then
    SynchroniseTreeViews(pCustom);
  if not FSynchronising then
    DisplayValues(pOriginal);
end;

procedure TfrmEditSetting2.SynchroniseTreeViews(inSelectedProfile : TProfile);
var
  parentNodes: TList;
  node, parentNode, otherNode, otherNodeChild: TTreeNode;
  selected, other : PProfileControls;
  n: Integer;
  foundMatch: Boolean;
begin
  RetrieveProfileControls(inSelectedProfile, selected, other);

  parentNodes := TList.Create;
  try
    { trace the parentage of the selected node back to the root }
    node := selected.TreeView.Selected;

    parentNodes.Add(node);
    parentNode := node.Parent;

    while Assigned(parentNode) do begin
      parentNodes.Add(parentNode);
      parentNode := parentNode.Parent;
    end; // while

    { trace down the parentage list to find the
    corresponding node in the other treeview.
    The top level parents will not match however
    all subsequent nodes should. }
    node := selected.TreeView.Items.GetFirstNode;
    otherNode := other.TreeView.Items.GetFirstNode;
    Assert(node = parentNodes[parentNodes.Count - 1]);

    n := parentNodes.Count - 2;
    foundMatch := True;
    while foundMatch and (n >= 0) do begin
      node := parentNodes[n];

      foundMatch := False;
      otherNodeChild := otherNode.GetFirstChild;
      while not foundMatch and Assigned(otherNodeChild) do begin
        if otherNodeChild.Text = node.Text then
          foundMatch := True
        else
          otherNodeChild := otherNode.GetNextChild(otherNodeChild);
      end; // while

      if foundMatch then
        otherNode := otherNodeChild;

      Dec(n);
    end; // while
  finally
    parentNodes.Free;
  end; // finally

  FSynchronising := True;
  try
    other.TreeView.Selected := otherNode;
    other.TreeView.Selected.Expand(False);
  finally
    FSynchronising := False;
  end; // finally
end;

procedure TfrmEditSetting2.RetrieveProfileControls(inPrimaryProfile : TProfile;
out outProfileControls1, outProfileControls2 : PProfileControls);
begin
  outProfileControls1 := @FProfile[inPrimaryProfile];
  case inPrimaryProfile of
    pOriginal: outProfileControls2 := @FProfile[pCustom];
    pCustom: outProfileControls2 := @FProfile[pOriginal];
  else
    raise EDSEException.Create('Unknown TProfile value in OtherProfile()');
  end; // case
end;

procedure TfrmEditSetting2.DisplayValues(inProfile : TProfile; inPreserveItemIndex : Boolean);
var
  selectedRegKey, otherRegKey : string;
  selected, other : PProfileControls;
  selectedValueNames, otherValueNames : TStringList;
  s, o, comparison, oldItemIndex : Integer;
  selectedValuePresent, otherValuePresent : Boolean;
begin
  RetrieveProfileControls(inProfile, selected, other);

  if inPreserveItemIndex then
    oldItemIndex := selected.ListBox.ItemIndex
  else
    oldItemIndex := -1;
  selected.ListBox.Clear;
  other.ListBox.Clear;

  selectedValueNames := TStringList.Create;
  otherValueNames := TStringList.Create;
  try
    { get the list of values for the selected treeview node }
    selectedRegKey := RegKeyForSelected(selected.TreeView.Selected);
    if not selected.Registry.OpenKey(selectedRegKey, False) then
      raise EDSEException.Create('Cannot open registry key ' + selectedRegKey);
    selected.Registry.GetValueNames(selectedValueNames);
    selectedValueNames.Sort;

    { get the list of values for the other treeview node
    if it matches the selected node, otherwise leave it empty }
    if NodesMatch(selected.TreeView.Selected, other.TreeView.Selected) then begin
      otherRegKey := RegKeyForSelected(other.TreeView.Selected);
      if not other.Registry.OpenKey(otherRegKey, False) then
        raise EDSEException.Create('Cannot open registry key ' + otherRegKey);
      other.Registry.GetValueNames(otherValueNames);
      otherValueNames.Sort;
    end; // if

    s := 0;
    o := 0;
    while (s < selectedValueNames.Count) or (o < otherValueNames.Count) do begin
      selectedValuePresent := False;
      otherValuePresent := False;

      { Determine which of the selected and other values is present }
      if (s < selectedValueNames.Count) and (o < otherValueNames.Count) then begin
        comparison := CompareText(selectedValueNames[s], otherValueNames[o]);
        if comparison <= 0 then
          selectedValuePresent := True;
        if comparison >= 0 then
          otherValuePresent := True;
      end
      else if (s < selectedValueNames.Count) and (o >= otherValueNames.Count) then
        selectedValuePresent := True
      else if (s >= selectedValueNames.Count) and (o < otherValueNames.Count) then
        otherValuePresent := True
      else
        Assert(False, 'Error in if/else statement ' + Format('s=%d s max=%d o=%d o max=%d', [s, selectedValueNames.Count, o, otherValueNames.Count]));

      { Add the present values to the listboxes }

      if selectedValuePresent then begin
        selected.ListBox.Items.Add(selectedValueNames[s]);
        Inc(s);
      end else
        selected.ListBox.Items.Add('');

      if otherValuePresent then begin
        other.ListBox.Items.Add(otherValueNames[o]);
        Inc(o);
      end else
        other.ListBox.Items.Add('');
    end; // while
  finally
    selectedValueNames.Free;
    otherValueNames.Free;
  end; // finally

  if inPreserveItemIndex then
    selected.ListBox.ItemIndex := oldItemIndex;
end;

function TfrmEditSetting2.NodesMatch(inSelectedNode, inOtherNode : TTreeNode) : Boolean;
var
  node1, node2: TTreeNode;
  matching: Boolean;
begin
  Assert(Assigned(inSelectedNode));
  Assert(Assigned(inOtherNode));

  node1 := inSelectedNode;
  node2 := inOtherNode;

  matching := True;
  if node1.Text <> node2.Text then
    matching := False;

  while matching and Assigned(node1) and Assigned(node2) do begin
    if node1.Text <> node2.Text then begin
      if not Assigned(node1.Parent) and not Assigned(node2.Parent) then
        matching := True { have reached top and nodes will always be different }
      else
        matching := False;
    end; // if

    node1 := node1.Parent;
    node2 := node2.Parent;
  end; // while

  Result := matching;
end;

function TfrmEditSetting2.RegKeyForSelected(inNode : TTreeNode) : string;
var
  node: TTreeNode;
begin
  Assert(Assigned(inNode));
  node := inNode;
  Result := node.Text;
  while Assigned(node.Parent) do begin
    Result := node.Parent.Text + '\' + Result;
    node := node.Parent;
  end; // while
  Result := '\' + Result;
end;

function TfrmEditSetting2.MatchingRegKeyForSelected(inSelectedProfile: TProfile) : string;
var
  node: TTreeNode;
  selected, other : PProfileControls;
begin
  RetrieveProfileControls(inSelectedProfile, selected, other);

  node := selected.TreeView.Selected;
  Result := '';
  repeat
    Result := node.Text + '\' + Result;
    node := node.Parent;
  until not Assigned(node.Parent);

  Result := '\' + other.TreeView.Items.GetFirstNode.Text + '\' + Result;
end;

procedure TfrmEditSetting2.DeleteValue(inSelectedProfile : TProfile);
var
  valueName: string;
  regKey: string;
  selected : PProfileControls;
begin
  selected := @FProfile[inSelectedProfile];

  regKey := RegKeyForSelected(selected.TreeView.Selected);
  if not selected.Registry.OpenKey(regKey, False) then
    raise EDSEException.Create('Unable to open reg key ' + regKey);

  valueName := selected.ListBox.Items[selected.ListBox.ItemIndex];
  if not selected.Registry.DeleteValue(valueName) then
    raise EDSEException.Create('Unable to delete value ' + valueName);
end;


procedure TfrmEditSetting2.SynchroniseListBoxes(inSelectedProfile : TProfile);
var
  selected, other : PProfileControls;
begin
  if not FSynchronising then begin
    FSynchronising := True;
    try
      RetrieveProfileControls(inSelectedProfile, selected, other);
      other.ListBox.ItemIndex := selected.ListBox.ItemIndex;
    finally
      FSynchronising := False;
    end; // finally
  end; // if
end;

procedure TfrmEditSetting2.lbOriginalClick(Sender: TObject);
begin
  SynchronizeAndDisplayValues;
end;

procedure TfrmEditSetting2.lbOriginalKeyPress(Sender: TObject; var Key: Char);
begin
  SynchronizeAndDisplayValues;
end;

procedure TfrmEditSetting2.lbOriginalDblClick(Sender: TObject);
begin
  SynchronizeAndDisplayValues;
end;

procedure TfrmEditSetting2.lbCustomClick(Sender: TObject);
begin
  SynchronizeAndDisplayValues;
end;

procedure TfrmEditSetting2.lbCustomDblClick(Sender: TObject);
begin
  SynchronizeAndDisplayValues;
end;

procedure TfrmEditSetting2.lbCustomKeyPress(Sender: TObject; var Key: Char);
begin
  SynchronizeAndDisplayValues;
end;

procedure TfrmEditSetting2.DisplayValue(inProfile : TProfile);
var
  regKey : string;
  valueName: string;
  selected : PProfileControls;
begin
  selected := @FProfile[inProfile];

  regKey := RegKeyForSelected(selected.TreeView.Selected);
  if not selected.Registry.OpenKey(regKey, False) then
    raise EDSEException.Create('Unable to open regKey ' + regKey);

  valueName := selected.ListBox.Items[selected.ListBox.ItemIndex];

  selected.Memo.Text := '';
  if valueName <> '' then begin
    case selected.Registry.GetDataType(valueName) of
      rdString:
        selected.Memo.Text := selected.Registry.ReadString(valueName);

      rdExpandString:
        selected.Memo.Text := selected.Registry.ReadString(valueName);

      rdInteger:
        selected.Memo.Text := IntToStr(selected.Registry.ReadInteger(valueName));

      rdBinary:
        selected.Memo.Text := '(Binary)';
    else
      selected.Memo.Text := '(Unknown)';
    end; // case
  end; // if
end;

procedure TfrmEditSetting2.actCopyRightExecute(Sender: TObject);
begin
  CopyValue(pOriginal);
  DisplayValues(pOriginal, True);
end;

procedure TfrmEditSetting2.actCopyLeftExecute(Sender: TObject);
begin
  if MessageBox(Self.Handle, PChar('You are about to overwrite registry values in your base profile. ' +
     'Are you sure you wish to do this?'), 'Confirm overwrite', MB_YESNO + MB_DEFBUTTON2 + MB_ICONQUESTION) = IDYES then begin
    CopyValue(pCustom);
    DisplayValues(pCustom, True);
  end; // if
end;

procedure TfrmEditSetting2.CopyValue(inSourceProfile : TProfile);
var
  sourceRegKey, destRegKey, valueName: string;
  buf : pointer;
  source, dest : PProfileControls;
  dataSize, bytesRead : integer;
begin
  RetrieveProfileControls(inSourceProfile, source, dest);

  sourceRegKey := RegKeyForSelected(source.TreeView.Selected);
  if not source.Registry.OpenKey(sourceRegKey, False) then
    raise EDSEException.Create('Unable to open reg key ' + sourceRegKey);

  destRegKey := MatchingRegKeyForSelected(inSourceProfile);
  if not dest.Registry.OpenKey(destRegKey, True) then
    raise EDSEException.Create('Unable to open/create reg key ' + destRegKey);

  valueName := source.ListBox.Items[source.ListBox.ItemIndex];

  case source.Registry.GetDataType(valueName) of
    rdString:
      dest.Registry.WriteString(valueName, source.Registry.ReadString(valueName));

    rdExpandString:
      dest.Registry.WriteExpandString(valueName, source.Registry.ReadString(valueName));

    rdInteger:
      dest.Registry.WriteInteger(valueName, source.Registry.ReadInteger(valueName));

    rdBinary:
      begin
        dataSize := source.Registry.GetDataSize(valueName);
        GetMem(buf, dataSize);
        try
          bytesRead := source.Registry.ReadBinaryData(valueName, buf^, dataSize);
          Assert(bytesRead = dataSize);
          dest.Registry.WriteBinaryData(valueName, buf^, dataSize);
        finally
          FreeMem(buf, dataSize);
        end; // finally
      end;

  else
    raise EDSEException.Create('Cannot copy registry value ' + valueName);
  end; // case
end;

procedure TfrmEditSetting2.actDeleteRightExecute(Sender: TObject);
begin
  DeleteValue(pCustom);
  DisplayValues(pCustom, True);
end;

procedure TfrmEditSetting2.actDeleteLeftExecute(Sender: TObject);
begin
  if MessageBox(Self.Handle, PChar('You are about to delete registry values in your base profile. ' +
     'Are you sure you wish to do this?'), 'Confirm deletion', MB_YESNO + MB_DEFBUTTON2 + MB_ICONQUESTION) = IDYES then begin
    DeleteValue(pOriginal);
    DisplayValues(pOriginal, True);
  end; // if
end;

procedure TfrmEditSetting2.TreeViewGetImageIndex(Sender: TObject; Node: TTreeNode);
begin
  Node.ImageIndex := 0;
end;

procedure TfrmEditSetting2.TreeViewGetSelectedIndex(Sender: TObject; Node: TTreeNode);
begin
  Node.SelectedIndex := 1;
end;

procedure TfrmEditSetting2.SetRegistryPath(const ADelphiPath, ASettingName,
  ACustomSettingPath: string);
begin
  FCustomSettingRegPath  := ACustomSettingPath;
  FDelphiRegPath         := ADelphiPath;
  lblCustom.Caption      := ASettingName;
end;

//Refactored and used by various action on the treeview.
procedure TfrmEditSetting2.SynchronizeAndDisplayValues;
begin
  SynchroniseListBoxes(pOriginal);
  DisplayValue(pOriginal);
  DisplayValue(pCustom);
end;

end.
