{-----------------------------------------------------------------------------
 Unit Name: fNewSetting
 Author:    Erwien Saputra
 Purpose:   User interface to add a new setting. If the class function returns
            true, the NewSettingName is meaningful.
 History:
 02/21/2005 - Updated the tab order.
-----------------------------------------------------------------------------}

unit fNewSetting;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls;

type
  TfrmNewSetting = class(TForm)
    lblSettingName: TLabeledEdit;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    cbCopyCurrentIDESetting: TCheckBox;
    procedure lblSettingNameKeyPress(Sender: TObject; var Key: Char);
    procedure lblSettingNameChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    class function Execute (var NewSettingName : string;
        var UseCurrentIDESetting: boolean) : boolean;
  end;

var
  frmNewSetting: TfrmNewSetting;

implementation

{$R *.dfm}
const
  DEFAULT_SETTING_HINT =
    'Check this if you want to use the default IDE setting to be copied to ' +
    'your new custom setting.' + sLineBreak +
    'If this option is unchecked, the new custom setting will have the ' +
    'default factory setting.';

{ TfrmNewSetting }
constructor TfrmNewSetting.Create(AOwner: TComponent);
begin
  inherited;
  cbCopyCurrentIDESetting.Hint := DEFAULT_SETTING_HINT;
end;

//Entry point for this form. If NewSettingName is initialized, it will be used
//on the edit box.
//This function returns true if the user clicks OK and the user set the new
//setting name.
class function TfrmNewSetting.Execute(var NewSettingName: string;
    var UseCurrentIDESetting: boolean) : boolean;
var
  frm : TfrmNewSetting;
begin
  frm := TfrmNewSetting.Create (nil);

  try
    if Trim (NewSettingName) <> EmptyStr then
      frm.lblSettingName.Text := NewSettingName
    else
      frm.lblSettingName.Text := EmptyStr;

    Result := (frm.ShowModal = MrOK) and
              (frm.lblSettingName.Text <> EmptyStr);

    UseCurrentIDESetting := frm.cbCopyCurrentIDESetting.Checked;

    if (Result = true) then
      NewSettingName := frm.lblSettingName.Text;

  finally
    frm.Release;
  end;
end;

procedure TfrmNewSetting.lblSettingNameChange(Sender: TObject);
begin
  btnOK.Enabled := not SameText (Trim (lblSettingName.Text), EmptyStr);
end;

procedure TfrmNewSetting.lblSettingNameKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = '\' then
    Key := #0;
end;

end.
